package com.uoc.textiller.config;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.config.annotation.authentication.builders.AuthenticationManagerBuilder;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;


/**
 * Configuració d'Spring Security
 * @author lrodriguezmos@uoc
 */
@Configuration
@EnableWebSecurity
public class SecurityConfig extends WebSecurityConfigurerAdapter {

    @Autowired
    private UserDetailsServiceImpl userDetailsService;

    @Bean
    public BCryptPasswordEncoder encoder() {
        return new BCryptPasswordEncoder();
    }

    //Variable per evitar Spring Security a altres recursos (no html)
    String[] resources = new String[]{
        "/css/**","/img/**","/js/**"
    };

    @Override
    protected void configure(AuthenticationManagerBuilder auth) throws Exception {       
        auth.userDetailsService(userDetailsService).passwordEncoder(encoder());
    }

    @Override
    protected void configure(HttpSecurity http) throws Exception {
        http
            .authorizeRequests()
            .antMatchers(resources).permitAll()
            .antMatchers("/","/index").permitAll()
            // gestió usuaris/materials/processos -> ADMINISTRADOR
            .antMatchers("/usuaris/**","/materials/**","/processos/**").hasRole("ADMINISTRADOR")
            // veure teixits -> tots els rols
            .antMatchers("/teixits/llista","/teixits/consultar","/teixits/cercar","/teixits/pag/**")
                .hasAnyRole("ADMINISTRADOR","MAGATZEM","TALLADOR","CONFECCIONISTA")
            // editar teixits -> ADMINISTRADOR, MAGATZEM, TALLADOR
            .antMatchers("/teixits/formulari","/teixits/guardar").hasAnyRole("ADMINISTRADOR","MAGATZEM","TALLADOR")
            // gestió teixits -> ADMINISTRADOR, MAGATZEM
            .antMatchers("/teixits/**").hasAnyRole("ADMINISTRADOR","MAGATZEM")
            // gestió proveïdors -> ADMINISTRADOR, MAGATZEM
            .antMatchers("/proveidors/**").hasAnyRole("ADMINISTRADOR","MAGATZEM")
            // veure fornitures -> ADMINISTRADOR, MAGATZEM, CONFECCIONISTA
            .antMatchers("/fornitures/llista","/fornitures/consultar","/fornitures/cercar","/fornitures/pag/**")    
                .hasAnyRole("ADMINISTRADOR","MAGATZEM","CONFECCIONISTA")
            // gestió fornitures -> ADMINISTRADOR, MAGATZEM
            .antMatchers("/fornitures/**").hasAnyRole("ADMINISTRADOR","MAGATZEM")
            // gestió escandalls -> ADMINISTRADOR, CONFECCIONISTA
            .antMatchers("/escandalls/**").hasAnyRole("ADMINISTRADOR","CONFECCIONISTA")
            // per la resta de peticions només cal estar identificat
            .anyRequest().authenticated()
            .and()
        .formLogin()
            .loginPage("/index").permitAll()
            .usernameParameter("nom")
            .defaultSuccessUrl("/inici")
            .failureUrl("/index?error=true")
            .loginProcessingUrl("/acces").permitAll()
            .and()
        .logout()
            .permitAll()
            .logoutSuccessUrl("/index?logout");
    }

}	