package com.uoc.textiller.controller;

import java.math.BigDecimal;
import java.util.List;

import javax.servlet.http.HttpSession;
import javax.validation.Valid;

import com.uoc.textiller.model.Escandall;
import com.uoc.textiller.model.EscandallFornitura;
import com.uoc.textiller.model.EscandallTeixit;
import com.uoc.textiller.model.EscandallProces;
import com.uoc.textiller.model.Usuari;
import com.uoc.textiller.service.EscandallService;
import com.uoc.textiller.service.FornituraService;
import com.uoc.textiller.service.ProcesService;
import com.uoc.textiller.service.TeixitService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;

/**
 * Controlador Escandall
 * @author lrodriguezmos@uoc
 */
@Controller
@RequestMapping("/escandalls")
public class EscandallController {

    /** variables per definir valors repetitius */

    private static final String ESCANDALL_ATTR = "escandall";
    private static final String ESCANDALLSLIST_ATTR = "escandallsList";
    private static final String ESCANDALLS_VIEW = "escandalls";
    private static final String ESCANDALL_VIEW = "escandall";
    private static final String ESCANDALLFORM_VIEW = "escandallForm";
    private static final String ESCANDALLS_REDIRECT = "redirect:/escandalls/llista";

    private static final String ESCANDALLTEIXIT_ATTR = "escandallTeixit";
    private static final String ESCANDALLTEIXITFORM_VIEW = "escandallTeixitForm";
    private static final String ESCANDALLFORNITURA_ATTR = "escandallFornitura";
    private static final String ESCANDALLFORNITURAFORM_VIEW = "escandallFornituraForm";
    private static final String ESCANDALLPROCES_ATTR = "escandallProces";
    private static final String ESCANDALLPROCESFORM_VIEW = "escandallProcesForm";
    private static final String ESCANDALL_REDIRECT = "redirect:/escandalls/consultar?id=";
    private static final String TEIXITSLIST_ATTR = "teixitsList";
    private static final String FORNITURESLIST_ATTR = "fornituresList";
    private static final String PROCESSOSLIST_ATTR = "processosList";
    
    @Autowired
    private EscandallService escandallService;

    @Autowired
    private TeixitService teixitService;

    @Autowired
    private FornituraService fornituraService;

    @Autowired
    private ProcesService procesService;
    

    @GetMapping({"","/"})
    public String escandallsRedirect() {

        return ESCANDALLS_REDIRECT;
    }

    @GetMapping("/llista")
    public String escandallsList(Model model) {

        List<Escandall> escandallsList = escandallService.getEscandallsList();
        model.addAttribute(ESCANDALLSLIST_ATTR, escandallsList);
        
        return findPaginated(1, model);
    }

    @GetMapping("/llistaUsuari")
    public String escandallsUsuariList(Model model, HttpSession session) {

        Usuari usuari = (Usuari) session.getAttribute("usuari");
        List<Escandall> escandallsList = escandallService.getEscandallsUsuariList(usuari);
        
        model.addAttribute(ESCANDALLSLIST_ATTR, escandallsList);
        model.addAttribute("escandallsSubtitol", "Usuari: "+usuari.getNom());
        
        return ESCANDALLS_VIEW;
    }

    @GetMapping("/pag/{pageNo}")
    public String findPaginated(@PathVariable(value = "pageNo") int pageNo, Model model) {
    
        int pageSize = 10;
    
        Page<Escandall> page = escandallService.findPaginated(pageNo, pageSize);
        List<Escandall> escandallsList = page.getContent();

        model.addAttribute("view", ESCANDALLS_VIEW);
        model.addAttribute("currentPage", pageNo);
        model.addAttribute("totalPages", page.getTotalPages());
        model.addAttribute("totalItems", page.getTotalElements());
        model.addAttribute(ESCANDALLSLIST_ATTR, escandallsList);

        return ESCANDALLS_VIEW;
    }

    @GetMapping("/cercar")
    public String search(Model model, String keyword) {
        if(keyword!=null) {
            List<Escandall> escandallsList = escandallService.findByKeyword(keyword);
            model.addAttribute(ESCANDALLSLIST_ATTR, escandallsList);
            model.addAttribute("escandallsSubtitol", "Paraula clau: "+keyword);
        } else {
            List<Escandall> escandallsList = escandallService.getEscandallsList();
            model.addAttribute(ESCANDALLSLIST_ATTR, escandallsList);
        }
        
        return ESCANDALLS_VIEW;
    }

    @GetMapping("/consultar")
    public String escandallView(@RequestParam(name = "id", required = true) Long id, Model model) {
        
        Escandall escandall = escandallService.findById(id);

        model.addAttribute(ESCANDALL_ATTR, escandall);

        BigDecimal costTotalMaterials = BigDecimal.valueOf(0);

        for (EscandallTeixit et : escandall.getTeixits()) {
            costTotalMaterials = costTotalMaterials.add(et.getTotal().multiply(et.getTeixit().getCost()));
        }

        for (EscandallFornitura ef : escandall.getFornitures()) {
            costTotalMaterials = costTotalMaterials.add(BigDecimal.valueOf(ef.getTotal()).multiply(ef.getFornitura().getCost()));
        }

        BigDecimal costTotalProcessos = BigDecimal.valueOf(0);

        for (EscandallProces ep : escandall.getProcessos()) {
            costTotalProcessos = costTotalProcessos.add(BigDecimal.valueOf(ep.getTotal()).multiply(ep.getProces().getCost()));
        }

        model.addAttribute("costTotalMaterials", costTotalMaterials.setScale(2));
        model.addAttribute("costTotalProcessos", costTotalProcessos.setScale(2));

        return ESCANDALL_VIEW;
    }

    @GetMapping("/formulari")
    public String escandallForm(@RequestParam(name = "id", required = true) Long id, HttpSession session, Model model) {

        Escandall escandall = new Escandall();

        if(id > 0) {
            escandall = escandallService.findById(id);
            model.addAttribute("titol", "Modificar escandall");
            model.addAttribute("subtitol", "");
        } else {
            model.addAttribute("titol", "Afegir escandall");
            model.addAttribute("subtitol", "Introdueix les dades del nou escandall");
            escandall.setUsuari((Usuari) session.getAttribute("usuari"));
        }

        model.addAttribute(ESCANDALL_ATTR, escandall);

        return ESCANDALLFORM_VIEW;
    }
    
    @PostMapping("/guardar")
    public String save(@RequestParam(name = "nom_anterior", required = true) String nom_anterior, 
                        @Valid Escandall escandall, BindingResult result, HttpSession session, Model model) {

        //en cas que modifiqui el nom (atribut únic a la base dades), verifiquem que no existeixi ja
        if(!escandall.getNom().equals(nom_anterior) && escandallService.existsNom(escandall.getNom())) {
            result.rejectValue("nom", "error.nom", "Aquest nom ja existeix a la base de dades");
            escandall.setNom(nom_anterior);
        }

        if(result!=null && result.hasErrors()) {

            if(escandall.getId()!=null && escandall.getId() != 0) {
                model.addAttribute("titol", "Modificar escandall");
                model.addAttribute("subtitol", "");
            } else {
                model.addAttribute("titol", "Afegir escandall");
                model.addAttribute("subtitol", "Introdueix les dades del nou escandall");
            }
            
            return ESCANDALLFORM_VIEW;
        } 

        if(escandall.getId()==null) {
            //l'usuari s'assigna automàticament a partir de l'usuari en sessió
            escandall.setUsuari((Usuari) session.getAttribute("usuari"));
        }
        
        Escandall e = escandallService.save(escandall);

        return ESCANDALL_REDIRECT+e.getId();
    }
    
    @GetMapping("/eliminar")
    public String remove(@RequestParam(name = "id", required = true) Long id, Model model) {
        
        escandallService.remove(id);
        model.addAttribute("successMsg", "S'ha eliminat l'escandall");

        List<Escandall> escandallsList = escandallService.getEscandallsList();
        model.addAttribute(ESCANDALLSLIST_ATTR, escandallsList);

        return ESCANDALLS_VIEW;
    }

    /****************************************** */
    /** Afegir o eliminar teixits a l'escandall */
    
    @GetMapping({"/afegirTeixit"})
    public String escandallTeixitForm(@RequestParam(name = "idEscandall", required = true) Long idEscandall, Model model) {

        EscandallTeixit escandallTeixit = new EscandallTeixit();
        Escandall escandall = escandallService.findById(idEscandall);

        model.addAttribute(ESCANDALL_ATTR, escandall);
        model.addAttribute(TEIXITSLIST_ATTR, teixitService.getTeixitsList());
        model.addAttribute(ESCANDALLTEIXIT_ATTR, escandallTeixit);

        return ESCANDALLTEIXITFORM_VIEW;
    }
    
    @PostMapping("/guardarTeixit")
    public String saveEscandallTeixit(@RequestParam(name = "idEscandall", required = true) Long idEscandall,
                            @Valid EscandallTeixit escandallTeixit, BindingResult result, Model model) {

        
        Escandall escandall = escandallService.findById(idEscandall);

        if(escandallService.hasTeixit(escandall.getTeixits(), escandallTeixit.getTeixit().getId())) {

            result.rejectValue("teixit", "error.teixit", 
                    "Aquest teixit ja està inclòs a l'escandall. Elimina'l i afegeix-lo de nou si vols modificar el total");
        }

        if(result!=null && result.hasErrors()) {
            model.addAttribute(ESCANDALL_ATTR, escandall);
            model.addAttribute(ESCANDALLTEIXIT_ATTR, escandallTeixit);
            model.addAttribute(TEIXITSLIST_ATTR, teixitService.getTeixitsList());
            return ESCANDALLTEIXITFORM_VIEW;
        }

        escandallTeixit.setTotal(escandallTeixit.getTotal().divide(BigDecimal.valueOf(100)));
        model.addAttribute(ESCANDALLTEIXIT_ATTR, escandallService.saveTeixit(escandall, escandallTeixit));
        
        return ESCANDALL_REDIRECT+escandall.getId();
    }
    
    @GetMapping("/eliminarTeixit")
    public String removeEscandallTeixit(@RequestParam(name = "id", required = true) Long id, 
                                    @RequestParam(name = "idEscandall", required = true) Long idEscandall, Model model) {
                
        escandallService.removeTeixit(id);
        model.addAttribute("successMsg", "S'ha eliminat el teixit d'aquest escandall");

        Escandall escandall = escandallService.findById(idEscandall);
        model.addAttribute(ESCANDALL_ATTR, escandall);

        return ESCANDALL_REDIRECT+escandall.getId();
    }

    /********************************************* */
    /** Afegir o eliminar fornitures a l'escandall */
    
    @GetMapping({"/afegirFornitura"})
    public String escandallFornituraForm(@RequestParam(name = "idEscandall", required = true) Long idEscandall, Model model) {

        EscandallFornitura escandallFornitura = new EscandallFornitura();
        Escandall escandall = escandallService.findById(idEscandall);

        model.addAttribute(ESCANDALL_ATTR, escandall);
        model.addAttribute(FORNITURESLIST_ATTR, fornituraService.getFornituresList());
        model.addAttribute(ESCANDALLFORNITURA_ATTR, escandallFornitura);

        return ESCANDALLFORNITURAFORM_VIEW;
    }
    
    @PostMapping("/guardarFornitura")
    public String saveEscandallFornitura(@RequestParam(name = "idEscandall", required = true) Long idEscandall,
                            @Valid EscandallFornitura escandallFornitura, BindingResult result, Model model) {

        
        Escandall escandall = escandallService.findById(idEscandall);

        if(escandallService.hasFornitura(escandall.getFornitures(), escandallFornitura.getFornitura().getId())) {

            result.rejectValue("fornitura", "error.fornitura", 
                    "Aquesta fornitura ja està inclosa a l'escandall. Elimina-la i afegeix-la de nou si vols modificar el total");
        }

        if(result!=null && result.hasErrors()) {
            model.addAttribute(ESCANDALL_ATTR, escandall);
            model.addAttribute(ESCANDALLFORNITURA_ATTR, escandallFornitura);
            model.addAttribute(FORNITURESLIST_ATTR, fornituraService.getFornituresList());
            return ESCANDALLFORNITURAFORM_VIEW;
        }
        
        model.addAttribute(ESCANDALLFORNITURA_ATTR, escandallService.saveFornitura(escandall, escandallFornitura));
        
        return ESCANDALL_REDIRECT+escandall.getId();
    }
    
    @GetMapping("/eliminarFornitura")
    public String removeEscandallFornitura(@RequestParam(name = "id", required = true) Long id, 
                                    @RequestParam(name = "idEscandall", required = true) Long idEscandall, Model model) {
                
        escandallService.removeFornitura(id);
        model.addAttribute("successMsg", "S'ha eliminat la fornitura d'aquest escandall");

        Escandall escandall = escandallService.findById(idEscandall);
        model.addAttribute(ESCANDALL_ATTR, escandall);

        return ESCANDALL_REDIRECT+escandall.getId();
    }

    /****************************************** */
    /** Afegir o eliminar processos a l'escandall */
    
    @GetMapping({"/afegirProces"})
    public String escandallProcesForm(@RequestParam(name = "idEscandall", required = true) Long idEscandall, Model model) {

        EscandallProces escandallProces = new EscandallProces();
        Escandall escandall = escandallService.findById(idEscandall);

        model.addAttribute(ESCANDALL_ATTR, escandall);
        model.addAttribute(PROCESSOSLIST_ATTR, procesService.getProcessosList());
        model.addAttribute(ESCANDALLPROCES_ATTR, escandallProces);

        return ESCANDALLPROCESFORM_VIEW;
    }
    
    @PostMapping("/guardarProces")
    public String saveEscandallProces(@RequestParam(name = "idEscandall", required = true) Long idEscandall,
                            @RequestParam(name = "hores", required = false) int hores,
                            @Valid EscandallProces escandallProces, BindingResult result, Model model) {

        
        Escandall escandall = escandallService.findById(idEscandall);

        if(escandallService.hasProces(escandall.getProcessos(), escandallProces.getProces().getId())) {

            result.rejectValue("proces", "error.proces", 
                    "Aquest procés ja està inclòs a l'escandall. Elimina'l i afegeix-lo de nou si vols modificar el total");
        }

        if(result!=null && result.hasErrors()) {
            model.addAttribute("hores", hores);
            model.addAttribute(ESCANDALL_ATTR, escandall);
            model.addAttribute(ESCANDALLPROCES_ATTR, escandallProces);
            model.addAttribute(PROCESSOSLIST_ATTR, procesService.getProcessosList());
            return ESCANDALLPROCESFORM_VIEW;
        }
        
        escandallProces.setTotal(escandallProces.getTotal()+(60*hores));
        model.addAttribute(ESCANDALLPROCES_ATTR, escandallService.saveProces(escandall, escandallProces));
        
        return ESCANDALL_REDIRECT+escandall.getId();
    }
    
    @GetMapping("/eliminarProces")
    public String removeEscandallProces(@RequestParam(name = "id", required = true) Long id, 
                                    @RequestParam(name = "idEscandall", required = true) Long idEscandall, Model model) {
                
        escandallService.removeProces(id);
        model.addAttribute("successMsg", "S'ha eliminat el procés d'aquest escandall");

        Escandall escandall = escandallService.findById(idEscandall);
        model.addAttribute(ESCANDALL_ATTR, escandall);

        return ESCANDALL_REDIRECT+escandall.getId();
    }

        
}
