package com.uoc.textiller.controller;

import java.util.List;

import javax.validation.Valid;

import com.uoc.textiller.model.Proces;
import com.uoc.textiller.service.ProcesService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;

/**
 * Controlador Proces
 * @author lrodriguezmos@uoc
 */
@Controller
@RequestMapping("/processos")
public class ProcesController {

    private static final String PROCES_ATTR = "proces";
    private static final String PROCESSOSLIST_ATTR = "processosList";
    private static final String PROCESSOS_VIEW = "processos";
    private static final String PROCESFORM_VIEW = "procesForm";
    private static final String PROCESSOS_REDIRECT = "redirect:/processos/llista";

    @Autowired
    private ProcesService procesService;

    @GetMapping({"","/"})
    public String processosRedirect() {
        
        return PROCESSOS_REDIRECT;
    }

    @GetMapping("/llista")
    public String processosList(Model model) {
        
        List<Proces> processosList = procesService.getProcessosList();
        
        model.addAttribute(PROCES_ATTR, processosList);

        return findPaginated(1, model);
    }

    @GetMapping("/pag/{pageNo}")
    public String findPaginated(@PathVariable(value = "pageNo") int pageNo, Model model) {
    
        int pageSize = 10;
    
        Page<Proces> page = procesService.findPaginated(pageNo, pageSize);
        List<Proces> usuarisList = page.getContent();

        model.addAttribute("view", PROCESSOS_VIEW);
        model.addAttribute("currentPage", pageNo);
        model.addAttribute("totalPages", page.getTotalPages());
        model.addAttribute("totalItems", page.getTotalElements());
        model.addAttribute(PROCESSOSLIST_ATTR, usuarisList);
        return PROCESSOS_VIEW;
    }

    @GetMapping("/cercar")
    public String search(Model model, String keyword) {
        if(keyword!=null) {
            List<Proces> processosList = procesService.findByKeyword(keyword);
            model.addAttribute(PROCESSOSLIST_ATTR, processosList);
        } else {
            List<Proces> processosList = procesService.getProcessosList();
            model.addAttribute(PROCESSOSLIST_ATTR, processosList);
        }
        
        return PROCESSOS_VIEW;
    }
    
    @GetMapping({"/formulari"})
    public String procesForm(@RequestParam(name = "id", required = true) Long id, Model model) {

        Proces proces = new Proces();

        if(id > 0){
            proces = procesService.findById(id);
            model.addAttribute("titol", "Modificar procés");
            model.addAttribute("subtitol", "");
        } else {
            model.addAttribute("titol", "Afegir procés");
            model.addAttribute("subtitol", "Introdueix el nom del nou procés:");
        }

        model.addAttribute(PROCES_ATTR, proces);

        return PROCESFORM_VIEW;
    }
    
    @PostMapping("/guardar")
    public String save(@RequestParam(name = "nom_anterior", required = true) String nom_anterior, 
                        @Valid Proces proces, BindingResult result, Model model) {

        //en cas que modifiqui el nom (atribut únic a la base dades), verifiquem que no existeixi ja
        if(!proces.getNom().equals(nom_anterior) && procesService.existsNom(proces.getNom())) {
            result.rejectValue("nom", "error.nom", "Aquest nom ja existeix a la base de dades");
            proces.setNom(nom_anterior);
        }

        if(result!=null && result.hasErrors()) {
            if(proces.getId()!=null && proces.getId() != 0) {
                model.addAttribute("titol", "Modificar procés");
                model.addAttribute("subtitol", "");
            } else {
                model.addAttribute("titol", "Afegir procés");
                model.addAttribute("subtitol", "Introdueix el nom del nou procés:");
            }
            return PROCESFORM_VIEW;
        } 
        
        model.addAttribute(PROCES_ATTR, procesService.save(proces));
        return PROCESSOS_REDIRECT;
    }
    
    @GetMapping("/eliminar")
    public String remove(@RequestParam(name = "id", required = true) Long id, Model model) {
        
        //no podem esborrar el procés si té escandalls associats
        if(procesService.hasEscandalls(id)){
            model.addAttribute("errorMsg", 
                "Aquest procés té escandalls associats.<br> "+
                "Si vols eliminar el procés, primer hauràs <br> "+
                "d'eliminar els escandalls que el contenen.");

        } else {
            procesService.remove(id);
            model.addAttribute("successMsg", "S'ha eliminat el procés");
        }
        
        List<Proces> processosList = procesService.getProcessosList();
        model.addAttribute(PROCESSOSLIST_ATTR, processosList);

        return PROCESSOS_VIEW;
    }
    
}
