package com.uoc.textiller.controller;

import java.util.List;

import javax.validation.Valid;

import com.uoc.textiller.model.Proveidor;
import com.uoc.textiller.service.ProveidorService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;

/**
 * Controlador Proveidor
 * @author lrodriguezmos@uoc
 */
@Controller
@RequestMapping("/proveidors")
public class ProveidorController {

    private static final String PROVEIDOR_ATTR = "proveidor";
    private static final String PROVEIDORSLIST_ATTR = "proveidorsList";
    private static final String PROVEIDORS_VIEW = "proveidors";
    private static final String PROVEIDOR_VIEW = "proveidor";
    private static final String PROVEIDORFORM_VIEW = "proveidorForm";
    private static final String PROVEIDORS_REDIRECT = "redirect:/proveidors/llista";
    
    @Autowired
    private ProveidorService proveidorService;


    @GetMapping({"","/"})
    public String proveidorsRedirect() {
        
        return PROVEIDORS_REDIRECT;
    }

    @GetMapping("/llista")
    public String proveidorsList(Model model) {

        List<Proveidor> proveidorsList = proveidorService.getProveidorsList();

        model.addAttribute(PROVEIDORSLIST_ATTR, proveidorsList);

        return findPaginated(1, model);
    }

    @GetMapping("/pag/{pageNo}")
    public String findPaginated(@PathVariable(value = "pageNo") int pageNo, Model model) {
    
        int pageSize = 10;
    
        Page<Proveidor> page = proveidorService.findPaginated(pageNo, pageSize);
        List<Proveidor> proveidorsList = page.getContent();

        model.addAttribute("view", PROVEIDORS_VIEW);
        model.addAttribute("currentPage", pageNo);
        model.addAttribute("totalPages", page.getTotalPages());
        model.addAttribute("totalItems", page.getTotalElements());
        model.addAttribute(PROVEIDORSLIST_ATTR, proveidorsList);
        return PROVEIDORS_VIEW;
    }

    @GetMapping("/cercar")
    public String search(Model model, String keyword) {
        if(keyword!=null) {
            List<Proveidor> proveidorsList = proveidorService.findByKeyword(keyword);
            model.addAttribute(PROVEIDORSLIST_ATTR, proveidorsList);
            model.addAttribute("proveidorsSubtitol", "Paraula clau: "+keyword);
        } else {
            List<Proveidor> proveidorsList = proveidorService.getProveidorsList();
            model.addAttribute(PROVEIDORSLIST_ATTR, proveidorsList);
        }
        
        return PROVEIDORS_VIEW;
    }

    @GetMapping("/consultar")
    public String proveidorView(@RequestParam(name = "id", required = true) Long id, Model model) {

        Proveidor proveidor = proveidorService.findById(id);

        model.addAttribute(PROVEIDOR_ATTR, proveidor);

        return PROVEIDOR_VIEW;
    }
    
    @GetMapping({"/formulari"})
    public String proveidorForm(@RequestParam(name = "id", required = true) Long id, Model model) {

        Proveidor proveidor = new Proveidor();

        if(id > 0){
            proveidor = proveidorService.findById(id);
            model.addAttribute("titol", "Modificar proveïdor");
            model.addAttribute("subtitol", "");
        } else {
            model.addAttribute("titol", "Afegir proveïdor");
            model.addAttribute("subtitol", "Introdueix les dades del nou proveïdor:");
        }

        model.addAttribute(PROVEIDOR_ATTR, proveidor);

        return PROVEIDORFORM_VIEW;
    }
    
    @PostMapping("/guardar")
    public String save(@RequestParam(name = "nom_anterior", required = true) String nom_anterior, 
                        @Valid Proveidor proveidor, BindingResult result, Model model) {

        //en cas que modifiqui el nom (atribut únic a la base dades), verifiquem que no existeixi ja
        if(!proveidor.getNom().equals(nom_anterior) && proveidorService.existsNom(proveidor.getNom())) {
            result.rejectValue("nom", "error.nom", "Aquest nom ja existeix a la base de dades");
            proveidor.setNom(nom_anterior);
        }

        if(result!=null && result.hasErrors()) {
            if(proveidor.getId()!=null && proveidor.getId() != 0) {
                model.addAttribute("titol", "Modificar proveïdor");
                model.addAttribute("subtitol", "");
            } else {
                model.addAttribute("titol", "Afegir proveïdor");
                model.addAttribute("subtitol", "Introdueix les dades del nou proveïdor:");
            }
            return PROVEIDORFORM_VIEW;
        } 
        
        model.addAttribute(PROVEIDOR_ATTR, proveidorService.save(proveidor));
        return PROVEIDORS_REDIRECT;
    }
    
    @GetMapping("/eliminar")
    public String remove(@RequestParam(name = "id", required = true) Long id, Model model) {
        
        //no podem esborrar el proveïdor si té teixits o fornitures associats
        if(proveidorService.hasTeixits(id)){
            model.addAttribute("errorMsg", 
                "Aquest proveïdor té teixits associats.\n"+
                "Si vols eliminar el proveïdor, primer hauràs d'eliminar els seus teixits.");
        } else if(proveidorService.hasFornitures(id)){
            model.addAttribute("errorMsg", 
                "Aquest proveïdor té fornitures associades.\n"+
                "Si vols eliminar el proveïdor, primer hauràs d'eliminar les seves fornitures.");
        } else {
            proveidorService.remove(id);
            model.addAttribute("successMsg", "S'ha eliminat el proveïdor");
        }

        List<Proveidor> proveidorsList = proveidorService.getProveidorsList();
        model.addAttribute(PROVEIDORSLIST_ATTR, proveidorsList);

        return PROVEIDORS_VIEW;
    }
}
