package com.uoc.textiller.controller;

import java.util.List;

import javax.validation.Valid;

import com.uoc.textiller.model.Composicio;
import com.uoc.textiller.model.Teixit;
import com.uoc.textiller.service.MaterialService;
import com.uoc.textiller.service.ProveidorService;
import com.uoc.textiller.service.TeixitService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;

/**
 * Controlador Teixit
 * @author lrodriguezmos@uoc
 */
@Controller
@RequestMapping("/teixits")
public class TeixitController {

    /** variables per definir valors repetitius */
    private static final String TEIXIT_ATTR = "teixit";
    private static final String TEIXITSLIST_ATTR = "teixitsList";
    private static final String TEIXITS_VIEW = "teixits";
    private static final String TEIXIT_VIEW = "teixit";
    private static final String TEIXITFORM_VIEW = "teixitForm";
    private static final String TEIXITS_REDIRECT = "redirect:/teixits/llista";
    private static final String COMPOSICIO_ATTR = "composicio";
    private static final String COMPOSICIOFORM_VIEW = "teixitComposicioForm";
    private static final String TEIXIT_REDIRECT = "redirect:/teixits/consultar?id=";
    private static final String PROVEIDORSLIST_ATTR = "proveidorsList";
    private static final String MATERIALSLIST_ATTR = "materialsList";

    @Autowired
    private TeixitService teixitService;

    @Autowired
    private ProveidorService proveidorService;

    @Autowired
    private MaterialService materialService;


    @GetMapping({"","/"})
    public String teixitsRedirect() {
        
        return TEIXITS_REDIRECT;
    }

    @GetMapping("/llista")
    public String teixitsList(Model model, @RequestParam(name = "idProveidor", required = false) Long idProveidor) {
        
        if(idProveidor != null) {
            List<Teixit> teixitsList = teixitService.getTeixitsListByProveidor(idProveidor);
            model.addAttribute(TEIXITSLIST_ATTR, teixitsList);
            model.addAttribute("teixitsSubtitol", proveidorService.findById(idProveidor).getNom());
            return TEIXITS_VIEW;
        } else {
            List<Teixit> teixitsList = teixitService.getTeixitsList();
            model.addAttribute(TEIXITSLIST_ATTR, teixitsList);
            return findPaginated(1, model);
        }
    }

    @GetMapping("/pag/{pageNo}")
    public String findPaginated(@PathVariable(value = "pageNo") int pageNo, Model model) {
    
        int pageSize = 10;
    
        Page<Teixit> page = teixitService.findPaginated(pageNo, pageSize);
        List<Teixit> teixitsList = page.getContent();

        model.addAttribute("view", TEIXITS_VIEW);
        model.addAttribute("currentPage", pageNo);
        model.addAttribute("totalPages", page.getTotalPages());
        model.addAttribute("totalItems", page.getTotalElements());
        model.addAttribute(TEIXITSLIST_ATTR, teixitsList);
        return TEIXITS_VIEW;
    }

    @GetMapping("/cercar")
    public String search(Model model, String keyword) {
        if(keyword!=null) {
            List<Teixit> teixitsList = teixitService.findByKeyword(keyword);
            model.addAttribute(TEIXITSLIST_ATTR, teixitsList);
            model.addAttribute("teixitsSubtitol", "Paraula clau: "+keyword);
        } else {
            List<Teixit> teixitsList = teixitService.getTeixitsList();
            model.addAttribute(TEIXITSLIST_ATTR, teixitsList);
        }
        
        return TEIXITS_VIEW;
    }

    @GetMapping("/consultar")
    public String teixitView(@RequestParam(name = "id", required = true) Long id, Model model) {

        Teixit teixit = teixitService.findById(id);

        model.addAttribute(TEIXIT_ATTR, teixit);

        if(teixit.getGramatge() != null) {
            model.addAttribute("gramatge", teixit.getGramatge()+" g/m<sup>2</sup>"); //unitats gramatge: g/m^2
        }
        
        return TEIXIT_VIEW;
    }
    
    @GetMapping("/formulari")
    public String teixitForm(@RequestParam(name = "id", required = true) Long id, 
                                @RequestParam(name = "idProveidor", required = false) Long idProveidor,
                                Model model) {

        Teixit teixit = new Teixit();
        
        model.addAttribute(PROVEIDORSLIST_ATTR, proveidorService.getProveidorsList());

        if(id > 0){
            teixit = teixitService.findById(id);
            model.addAttribute("titol", "Modificar teixit");
            model.addAttribute("subtitol", "");
        } else {
            model.addAttribute("titol", "Afegir teixit");
            model.addAttribute("subtitol", "Introdueix les dades del nou teixit:");
            if(idProveidor != null) {
                // quan s'afegeix el teixit des del proveïdor
                teixit.setProveidor(proveidorService.findById(idProveidor));
            }
            
        }

        model.addAttribute(TEIXIT_ATTR, teixit);

        return TEIXITFORM_VIEW;
    }
    
    @PostMapping("/guardar")
    public String save(@RequestParam(name = "nom_anterior", required = true) String nom_anterior, 
                        @Valid Teixit teixit, BindingResult result, Model model) {

        //en cas que modifiqui el nom (atribut únic a la base dades), verifiquem que no existeixi ja
        if(!teixit.getNom().equals(nom_anterior) && teixitService.existsNom(teixit.getNom())) {
            result.rejectValue("nom", "error.nom", "Aquest nom ja existeix a la base de dades");
            teixit.setNom(nom_anterior);
        }

        if(result!=null && result.hasErrors()) {
        
            model.addAttribute(PROVEIDORSLIST_ATTR, proveidorService.getProveidorsList());

            if(teixit.getId()!=null && teixit.getId() != 0) {
                model.addAttribute("titol", "Modificar teixit");
                model.addAttribute("subtitol", "");
            } else {
                model.addAttribute("titol", "Afegir teixit");
                model.addAttribute("subtitol", "Introdueix les dades del nou teixit:");
            }
            
            return TEIXITFORM_VIEW;
        } 
        
        Teixit t = teixitService.save(teixit);
        model.addAttribute(TEIXIT_ATTR, t);
        
        return TEIXIT_REDIRECT+t.getId();
    }
    
    @GetMapping("/eliminar")
    public String remove(@RequestParam(name = "id", required = true) Long id, Model model) {
        
        //no podem esborrar el teixit si té escandalls associats
        if(teixitService.hasEscandalls(id)){
            model.addAttribute("errorMsg", 
                "Aquest teixit té escandalls associats.\n"+
                "Si vols eliminar el teixit, primer hauràs d'eliminar els escandalls que l'inclouen.");

        } else {
            teixitService.remove(id);
            model.addAttribute("successMsg", "S'ha eliminat el teixit");
        }

        List<Teixit> teixitsList = teixitService.getTeixitsList();
        model.addAttribute(TEIXITSLIST_ATTR, teixitsList);

        return TEIXITS_VIEW;
    }


    /***************************************** */
    /** Afegir o eliminar materials del teixit */
    
    @GetMapping({"/afegirComposicio"})
    public String composicioForm(@RequestParam(name = "idTeixit", required = true) Long idTeixit, Model model) {

        Composicio composicio = new Composicio();
        Teixit teixit = teixitService.findById(idTeixit);

        model.addAttribute(TEIXIT_ATTR, teixit);
        model.addAttribute(MATERIALSLIST_ATTR, materialService.getMaterialsList());
        model.addAttribute(COMPOSICIO_ATTR, composicio);

        return COMPOSICIOFORM_VIEW;
    }
    
    @PostMapping("/guardarComposicio")
    public String saveComposicio(@RequestParam(name = "idTeixit", required = true) Long idTeixit,
                            @Valid Composicio composicio, BindingResult result, Model model) {

        
        Teixit teixit = teixitService.findById(idTeixit);

        if(teixitService.hasMaterial(teixit.getMaterialsComposicio(), 
                                        composicio.getMaterial().getId())) {

            result.rejectValue("material", "error.material", 
                    "Aquest material ja està inclòs la composició del teixit");
        }

        if(result!=null && result.hasErrors()) {
            model.addAttribute(TEIXIT_ATTR, teixit);
            model.addAttribute(COMPOSICIO_ATTR, composicio);
            model.addAttribute(MATERIALSLIST_ATTR, materialService.getMaterialsList());
            return COMPOSICIOFORM_VIEW;
        }
        
        composicio.setId(null);
        model.addAttribute(COMPOSICIO_ATTR, teixitService.saveComposicio(teixit, composicio));
        
        return TEIXIT_REDIRECT+teixit.getId();
    }
    
    @GetMapping("/eliminarComposicio")
    public String removeComposicio(@RequestParam(name = "id", required = true) Long id, 
                                    @RequestParam(name = "idTeixit", required = true) Long idTeixit, Model model) {
                
        teixitService.removeComposicio(id);
        model.addAttribute("successMsg", "S'ha eliminat la composicio");

        Teixit teixit = teixitService.findById(idTeixit);
        model.addAttribute(TEIXIT_ATTR, teixit);

        return TEIXIT_REDIRECT+teixit.getId();
    }

}
