package com.uoc.textiller.model;

import java.sql.Date;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.Size;

import org.hibernate.annotations.CreationTimestamp;
import org.hibernate.annotations.OnDelete;
import org.hibernate.annotations.OnDeleteAction;

/**
 * Entitat Escandall
 * @author lrodriguezmos@uoc
 */
@Entity
@Table(name = "escandalls")
public class Escandall {
    
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Long id;
    
    @NotEmpty(message = "El camp nom no pot estar buit")
    @Size(max = 32, message = "El camp nom no pot tenir més de 32 caràcters")
    @Column(length = 32, nullable = false, unique = true)
    private String nom;

    @Size(max = 255, message = "El camp descripció no pot tenir més de 255 caràcters")
    @Column(length = 255)
    private String descripcio;

    @Column(length = 8)
    private String referencia;

    @CreationTimestamp
    @Column(nullable = false)
    private Date data;

    @ManyToOne(optional = false)
    @JoinColumn(name = "id_usuari")
    private Usuari usuari;

    @OneToMany(mappedBy = "escandall")
    @OnDelete(action = OnDeleteAction.CASCADE)
    private List<EscandallTeixit> teixits;

    @OneToMany(mappedBy = "escandall")
    @OnDelete(action = OnDeleteAction.CASCADE)
    private List<EscandallFornitura> fornitures;

    @OneToMany(mappedBy = "escandall")
    @OnDelete(action = OnDeleteAction.CASCADE)
    private List<EscandallProces> processos;

    public Escandall() {
    }

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getNom() {
        return nom;
    }

    public void setNom(String nom) {
        this.nom = nom.trim();
    }

    public String getDescripcio() {
        return descripcio;
    }

    public void setDescripcio(String descripcio) {
        this.descripcio = descripcio.trim();
    }

    public String getReferencia() {
        return referencia;
    }

    public void setReferencia(String referencia) {
        this.referencia = referencia;
    }

    public Date getData() {
        return data;
    }

    public void setData(Date data) {
        this.data = data;
    }

    public Usuari getUsuari() {
        return usuari;
    }

    public void setUsuari(Usuari usuari) {
        this.usuari = usuari;
    }

    public List<EscandallTeixit> getTeixits() {
        return teixits;
    }

    public void setTeixits(List<EscandallTeixit> teixits) {
        this.teixits = teixits;
    }

    public List<EscandallFornitura> getFornitures() {
        return fornitures;
    }

    public void setFornitures(List<EscandallFornitura> fornitures) {
        this.fornitures = fornitures;
    }

    public List<EscandallProces> getProcessos() {
        return processos;
    }

    public void setProcessos(List<EscandallProces> processos) {
        this.processos = processos;
    }

}
