package com.uoc.textiller.model;

import java.math.BigDecimal;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

/**
 * Entitat Fornitura
 * @author lrodriguezmos@uoc
 */
@Entity
@Table(name = "fornitures")
public class Fornitura {
    
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Long id;
    
    @NotEmpty(message = "El camp nom no pot estar buit")
    @Size(max = 32, message = "El camp nom no pot tenir més de 32 caràcters")
    @Column(length = 32, nullable = false, unique = true)
    private String nom;

    @Size(max = 255, message = "El camp descripció no pot tenir més de 255 caràcters")
    @Column(length = 255)
    private String descripcio;

    @Column(length = 8)
    private String referencia;

    @NotNull(message = "Indica l''estoc")
    @Column(nullable = false)
    private int estoc;

    @Size(max = 45, message = "El camp ubicació no pot tenir més de 45 caràcters")
    @Column(length = 45)
    private String ubicacio;

    @NotNull(message = "Indica el cost per unitat")
    @Column(precision=6, scale=2)
    private BigDecimal cost;

    @ManyToOne(optional = false)
    @JoinColumn(name = "id_proveidor")
    private Proveidor proveidor;

    @OneToMany(mappedBy = "fornitura")
    private List<EscandallFornitura> escandalls;

    public Fornitura() {
    }

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getNom() {
        return nom;
    }

    public void setNom(String nom) {
        this.nom = nom.trim();
    }

    public String getDescripcio() {
        return descripcio;
    }

    public void setDescripcio(String descripcio) {
        this.descripcio = descripcio.trim();
    }

    public String getReferencia() {
        return referencia;
    }

    public void setReferencia(String referencia) {
        this.referencia = referencia;
    }

    public int getEstoc() {
        return estoc;
    }

    public void setEstoc(int estoc) {
        this.estoc = estoc;
    }

    public String getUbicacio() {
        return ubicacio;
    }

    public void setUbicacio(String ubicacio) {
        this.ubicacio = ubicacio.trim();
    }

    public BigDecimal getCost() {
        return cost;
    }

    public void setCost(BigDecimal cost) {
        this.cost = cost;
    }

    public Proveidor getProveidor() {
        return proveidor;
    }

    public void setProveidor(Proveidor proveidor) {
        this.proveidor = proveidor;
    }

    public List<EscandallFornitura> getEscandalls() {
        return escandalls;
    }

    public void setEscandalls(List<EscandallFornitura> escandalls) {
        this.escandalls = escandalls;
    }

}
