package com.uoc.textiller.model;

import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.Size;

/**
 * Entitat Proveidor
 * @author lrodriguezmos@uoc
 */
@Entity
@Table(name = "proveidors")
public class Proveidor {
    
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Long id;

    @NotEmpty(message = "El camp nom no pot estar buit")
    @Size(max = 32, message = "El camp nom no pot tenir més de 32 caràcters")
    @Column(length = 32, nullable = false, unique = true)
    private String nom;

    @Size(max = 255, message = "El camp descripció no pot tenir més de 255 caràcters")
    @Column(length = 255)
    private String descripcio;

    @Size(max = 255, message = "El camp localització no pot tenir més de 255 caràcters")
    @Column(length = 255)
    private String localitzacio;

    @NotEmpty(message = "El camp telèfon no pot estar buit")
    @Size(max = 45, message = "El camp telèfon no pot tenir més de 45 caràcters")
    @Column(length = 45)
    private String telefon;

    @NotEmpty(message = "El camp email no pot estar buit")
    @Size(max = 255, message = "El camp email no pot tenir més de 255 caràcters")
    @Column(length = 255)
    private String email;

    @OneToMany(mappedBy = "proveidor")
    private List<Teixit> teixits;

    @OneToMany(mappedBy = "proveidor")
    private List<Fornitura> fornitures;

    public Proveidor() {
    }

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getNom() {
        return nom;
    }

    public void setNom(String nom) {
        this.nom = nom.trim();
    }

    public String getDescripcio() {
        return descripcio;
    }

    public void setDescripcio(String descripcio) {
        this.descripcio = descripcio.trim();
    }

    public String getLocalitzacio() {
        return localitzacio;
    }

    public void setLocalitzacio(String localitzacio) {
        this.localitzacio = localitzacio.trim();
    }

    public String getTelefon() {
        return telefon;
    }

    public void setTelefon(String telefon) {
        this.telefon = telefon.replaceAll(" ", "");
    }

    public String getEmail() {
        return email;
    }

    public void setEmail(String email) {
        this.email = email.trim();
    }

    public List<Teixit> getTeixits() {
        return teixits;
    }

    public void setTeixits(List<Teixit> teixits) {
        this.teixits = teixits;
    }

    public List<Fornitura> getFornitures() {
        return fornitures;
    }

    public void setFornitures(List<Fornitura> fornitures) {
        this.fornitures = fornitures;
    }
    
}
