package com.uoc.textiller.model;

import java.math.BigDecimal;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

import com.uoc.textiller.model.enums.LligatsTeixit;

import org.hibernate.annotations.OnDelete;
import org.hibernate.annotations.OnDeleteAction;

/**
 * Entitat Teixit
 * @author lrodriguezmos@uoc
 */
@Entity
@Table(name = "teixits")
public class Teixit {
    
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Long id;
    
    @NotEmpty(message = "El camp nom no pot estar buit")
    @Size(max = 32, message = "El camp nom no pot tenir més de 32 caràcters")
    @Column(length = 32, nullable = false, unique = true)
    private String nom;

    @Size(max = 255, message = "El camp descripció no pot tenir més de 255 caràcters")
    @Column(length = 255)
    private String descripcio;

    @Column(length = 8)
    private String referencia;

    @NotNull(message = "Indica l''estoc")
    @Column(precision=8, scale=2)
    private BigDecimal estoc;

    @Column(precision=8, scale=2)
    private BigDecimal gramatge;

    @NotNull(message = "Indica l''ample")
    @Column(precision=4, scale=2)
    private BigDecimal ample;

    @NotNull(message = "Has de seleccionar un lligat")
    @Enumerated(EnumType.STRING)
    @Column(length = 25, nullable = false)
    private LligatsTeixit lligat;

    @Size(max = 45, message = "El camp ubicació no pot tenir més de 45 caràcters")
    @Column(length = 45)
    private String ubicacio;

    @NotNull(message = "Indica el cost per metre")
    @Column(precision=6, scale=2)
    private BigDecimal cost;

    @ManyToOne(optional = false)
    @JoinColumn(name = "id_proveidor")
    private Proveidor proveidor;

    @OneToMany(mappedBy="teixit")
    @OnDelete(action = OnDeleteAction.CASCADE)
    private List<Composicio> materialsComposicio;

    @OneToMany(mappedBy = "teixit")
    private List<EscandallTeixit> escandalls;

    public Teixit() {
    }

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getNom() {
        return nom;
    }

    public void setNom(String nom) {
        this.nom = nom.trim();
    }

    public String getDescripcio() {
        return descripcio;
    }

    public void setDescripcio(String descripcio) {
        this.descripcio = descripcio.trim();
    }

    public String getReferencia() {
        return referencia;
    }

    public void setReferencia(String referencia) {
        this.referencia = referencia;
    }

    public BigDecimal getEstoc() {
        return estoc;
    }

    public void setEstoc(BigDecimal estoc) {
        this.estoc = estoc;
    }

    public BigDecimal getGramatge() {
        return gramatge;
    }

    public void setGramatge(BigDecimal gramatge) {
        this.gramatge = gramatge;
    }

    public BigDecimal getAmple() {
        return ample;
    }

    public void setAmple(BigDecimal ample) {
        this.ample = ample;
    }

    public LligatsTeixit getLligat() {
        return lligat;
    }

    public void setLligat(LligatsTeixit lligat) {
        this.lligat = lligat;
    }

    public String getUbicacio() {
        return ubicacio;
    }

    public void setUbicacio(String ubicacio) {
        this.ubicacio = ubicacio.trim();
    }

    public BigDecimal getCost() {
        return cost;
    }

    public void setCost(BigDecimal cost) {
        this.cost = cost;
    }

    public Proveidor getProveidor() {
        return proveidor;
    }

    public void setProveidor(Proveidor proveidor) {
        this.proveidor = proveidor;
    }

    public List<Composicio> getMaterialsComposicio() {
        return materialsComposicio;
    }

    public void setMaterialsComposicio(List<Composicio> materialsComposicio) {
        this.materialsComposicio = materialsComposicio;
    }

    public List<EscandallTeixit> getEscandalls() {
        return escandalls;
    }

    public void setEscandalls(List<EscandallTeixit> escandalls) {
        this.escandalls = escandalls;
    }

}
