package com.uoc.textiller.model;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

import com.uoc.textiller.model.enums.EstatsUsuari;
import com.uoc.textiller.model.enums.RolsUsuari;

import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;

/**
 * Entitat Usuari que implementa UserDetails (Spring Security)
 * @author lrodriguezmos@uoc
 */
@Entity
@Table(name = "usuaris")
public class Usuari implements UserDetails{
    
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Long id;

    @NotEmpty(message = "El camp nom no pot estar buit")
    @Size(max = 16, message = "El camp nom no pot tenir més de 16 caràcters")
    @Column(length = 16, nullable = false, unique = true)
    private String nom;
    
    @NotEmpty(message = "El camp password no pot estar buit")
    @Column(length = 100, nullable = false)
    private String password;

    @NotNull(message = "Has de seleccionar un rol")
    @Enumerated(EnumType.STRING)
    @Column(length = 25, nullable = false)
    private RolsUsuari rol;

    @NotNull(message = "Has de seleccionar un estat")
    @Enumerated(EnumType.STRING)
    @Column(length = 10, nullable = false)
    private EstatsUsuari estat;

    @OneToMany(mappedBy = "usuari")
    private List<Escandall> escandalls;

    public Usuari() {
    }

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getNom() {
        return nom;
    }

    public void setNom(String nom) {
        this.nom = nom;
    }

    public String getPassword() {
        return password;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    public RolsUsuari getRol() {
        return rol;
    }

    public void setRol(RolsUsuari rol) {
        this.rol = rol;
    }

    public EstatsUsuari getEstat() {
        return estat;
    }

    public void setEstat(EstatsUsuari estat) {
        this.estat = estat;
    }

    public List<Escandall> getEscandalls() {
        return escandalls;
    }

    public void setEscandalls(List<Escandall> escandalls) {
        this.escandalls = escandalls;
    }

    @Override
    //mapejem rols de l'usuari amb Authorities de UserDetails
    public Collection<? extends GrantedAuthority> getAuthorities() {
        List<GrantedAuthority> rols = new ArrayList<>();
        rols.add(new SimpleGrantedAuthority(rol.toString()));
        return rols;
    }

    @Override
    public String getUsername() {
        return this.getNom();
    }

    @Override
    public boolean isAccountNonExpired() {
        return true;
    }

    @Override
    public boolean isAccountNonLocked() {
        return true;
    }

    @Override
    public boolean isCredentialsNonExpired() {
        return true;
    }

    @Override
    //utilitzem l'atribut "estat" per definir isEnabled() de UserDetails
    public boolean isEnabled() {
        return this.getEstat().equals(EstatsUsuari.ACTIU);
    }
  
}
