package com.uoc.textiller.service;

import java.util.List;

import com.uoc.textiller.model.Fornitura;
import com.uoc.textiller.repository.FornituraRepository;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;

/**
 * Implementació dels mètodes de FornituraService
 * @author lrodriguezmos@uoc
 */
@Service
public class FornituraServiceImpl implements FornituraService {

    @Autowired
    FornituraRepository fornituraRepository;

    public FornituraServiceImpl(FornituraRepository fornituraRepository) {
        this.fornituraRepository = fornituraRepository;
    }

    /** Cerca per ID */
    @Override
    public Fornitura findById(Long id) {
        return fornituraRepository.findById(id).get();
    }

    /** Cerca per nom */
    @Override
    public Fornitura findByNom(String nom) {
        return fornituraRepository.findByNom(nom);
    }

    /** Llista de fornitures */
    @Override
    public List<Fornitura> getFornituresList() {
        return fornituraRepository.findAll();
    }

    /** Llista de fornitures d'un proveïdor */
    @Override
    public List<Fornitura> getFornituresListByProveidor(Long idProveidor) {
        return fornituraRepository.findByProveidor(idProveidor);
    }

    /** Afegir o modificar fornitura */
    @Override
    public Fornitura save(Fornitura fornitura) {

        Fornitura f = fornituraRepository.save(fornitura);

        if(f.getReferencia() == null && f.getId() > 0) {
            //referència autogenerada a partir de l'identificador
            String referencia = "F" + String.format("%05d", f.getId());
            f.setReferencia(referencia);
            return fornituraRepository.save(f);
        }

        return f;
        
    }

    /** Eliminar fornitura */
    @Override
    public void remove(Long id) {
        fornituraRepository.deleteById(id);        
    }

    /** Booleà que indica si ja existeix el nom de la fornitura */
    @Override
    public boolean existsNom(String nom) {
        return fornituraRepository.findByNom(nom)!=null;
    }

    /** Cerca a la taula */
    @Override
    public List<Fornitura> findByKeyword(String keyword) {
        return fornituraRepository.findByKeyword(keyword);
    }

    /** Paginació de la taula */
    @Override
    public Page<Fornitura> findPaginated(int pageNo, int pageSize) {
        Pageable pageable = PageRequest.of(pageNo - 1, pageSize);
        return this.fornituraRepository.findAll(pageable);
    }       

    /** Booleà que indica si la fornitura té escandalls associats */
    @Override
    public boolean hasEscandalls(Long id) {
        return !findById(id).getEscandalls().isEmpty();
    } 
    
}
