package com.uoc.textiller.service;

import java.util.List;

import com.uoc.textiller.model.Material;
import com.uoc.textiller.repository.MaterialRepository;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;

/**
 * Implementació dels mètodes de MaterialService
 * @author lrodriguezmos@uoc
 */
@Service
public class MaterialServiceImpl implements MaterialService {
    
    @Autowired
    MaterialRepository materialRepository;

    public MaterialServiceImpl(MaterialRepository materialRepository) {
        this.materialRepository = materialRepository;
    }

    /** Cerca per ID */
    @Override
    public Material findById(Long id) {
        return materialRepository.findById(id).get();
    }

    /** Cerca per nom */
    @Override
    public Material findByNom(String nom) {
        return materialRepository.findByNom(nom);
    }

    /** Llista de materials */
    @Override
    public List<Material> getMaterialsList() {
        return materialRepository.findAll();
    }

    /** Afegir o modificar material */
    @Override
    public Material save(Material material) {
        return materialRepository.save(material);
    }

    /** Eliminar material */
    @Override
    public void remove(Long id) {
        materialRepository.deleteById(id);
        
    }

    /** Booleà que indica si ja existeix el nom del material */
    @Override
    public boolean existsNom(String nom) {
        return materialRepository.findByNom(nom)!=null;
    }

    /** Cerca a la taula */
    @Override
    public List<Material> findByKeyword(String keyword) {
        return materialRepository.findByKeyword(keyword);
    }

    /** Paginació de la taula */
    @Override
    public Page<Material> findPaginated(int pageNo, int pageSize) {
        Pageable pageable = PageRequest.of(pageNo - 1, pageSize);
        return this.materialRepository.findAll(pageable);
    }

    /** Booleà que indica si el material té composicions associades */
    @Override
    public boolean hasComposicions(Long id) {
        return !findById(id).getComposicions().isEmpty();
    }

    
}
