package com.uoc.textiller.service;

import java.util.List;

import com.uoc.textiller.model.Proces;
import com.uoc.textiller.repository.ProcesRepository;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;

/**
 * Implementació dels mètodes de ProcesService
 * @author lrodriguezmos@uoc
 */
@Service
public class ProcesServiceImpl implements ProcesService {
    
    @Autowired
    ProcesRepository procesRepository;

    public ProcesServiceImpl(ProcesRepository procesRepository) {
        this.procesRepository = procesRepository;
    }

    /** Cerca per ID */
    @Override
    public Proces findById(Long id) {
        return procesRepository.findById(id).get();
    }

    /** Cerca per nom */
    @Override
    public Proces findByNom(String nom) {
        return procesRepository.findByNom(nom);
    }

    /** Llista de processos */
    @Override
    public List<Proces> getProcessosList() {
        return procesRepository.findAll();
    }

    /** Afegir o modificar procés */
    @Override
    public Proces save(Proces proces) {
        return procesRepository.save(proces);
    }

    /** Eliminar procés */
    @Override
    public void remove(Long id) {
        procesRepository.deleteById(id);
        
    }

    /** Booleà que indica si ja existeix el nom del procés */
    @Override
    public boolean existsNom(String nom) {
        return procesRepository.findByNom(nom)!=null;
    }

    /** Cerca a la taula */
    @Override
    public List<Proces> findByKeyword(String keyword) {
        return procesRepository.findByKeyword(keyword);
    }

    /** Paginació de la taula */
    @Override
    public Page<Proces> findPaginated(int pageNo, int pageSize) {
        Pageable pageable = PageRequest.of(pageNo - 1, pageSize);
        return this.procesRepository.findAll(pageable);
    }

    /** Booleà que indica si el procés té escandalls associats */
    @Override
    public boolean hasEscandalls(Long id) {
        return !findById(id).getEscandalls().isEmpty();
    }

    
}
