package com.uoc.textiller.service;

import java.util.List;

import com.uoc.textiller.model.Proveidor;
import com.uoc.textiller.repository.ProveidorRepository;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;

/**
 * Implementació dels mètodes de ProveidorService
 * @author lrodriguezmos@uoc
 */
@Service
public class ProveidorServiceImpl implements ProveidorService {
    
    @Autowired
    ProveidorRepository proveidorRepository;

    public ProveidorServiceImpl(ProveidorRepository proveidorRepository) {
        this.proveidorRepository = proveidorRepository;
    }

    /** Cerca per ID */
    @Override
    public Proveidor findById(Long id) {
        return proveidorRepository.findById(id).get();
    }

    /** Cerca per nom */
    @Override
    public Proveidor findByNom(String nom) {
        return proveidorRepository.findByNom(nom);
    }

    /** Llista de proveïdors */
    @Override
    public List<Proveidor> getProveidorsList() {

        return proveidorRepository.findAll();
    }

    /** Afegir o modificar proveïdor */
    @Override
    public Proveidor save(Proveidor proveidor) {
        return proveidorRepository.save(proveidor);
    }

    /** Eliminar proveïdor */
    @Override
    public void remove(Long id) {
        proveidorRepository.deleteById(id);        
    }

    /** Booleà que indica si ja existeix el nom de proveïdor */
    @Override
    public boolean existsNom(String nom) {
        return proveidorRepository.findByNom(nom)!=null;
    }

    /** Booleà que indica si el proveïdor té teixits associats */
    @Override
    public boolean hasTeixits(Long id) {
        return !findById(id).getTeixits().isEmpty();
    }

    /** Booleà que indica si el proveïdor té fornitures associades */
    @Override
    public boolean hasFornitures(Long id) {
        return !findById(id).getFornitures().isEmpty();
    }

    
    /** Cerca a la taula */
    @Override
    public List<Proveidor> findByKeyword(String keyword) {
        return proveidorRepository.findByKeyword(keyword);
    }

    /** Paginació de la taula */
    @Override
    public Page<Proveidor> findPaginated(int pageNo, int pageSize) {
        Pageable pageable = PageRequest.of(pageNo - 1, pageSize);
        return this.proveidorRepository.findAll(pageable);
    }
    
}
