package com.uoc.textiller.service;

import java.util.List;

import com.uoc.textiller.model.Composicio;
import com.uoc.textiller.model.Teixit;
import com.uoc.textiller.repository.ComposicioRepository;
import com.uoc.textiller.repository.TeixitRepository;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;

/**
 * Implementació dels mètodes de TeixitService
 * @author lrodriguezmos@uoc
 */
@Service
public class TeixitServiceImpl implements TeixitService {

    @Autowired
    TeixitRepository teixitRepository;

    @Autowired
    ComposicioRepository composicioRepository;

    public TeixitServiceImpl(TeixitRepository teixitRepository) {
        this.teixitRepository = teixitRepository;
    }

    /** Cerca per ID */
    @Override
    public Teixit findById(Long id) {
        return teixitRepository.findById(id).get();
    }

    /** Cerca per nom */
    @Override
    public Teixit findByNom(String nom) {
        return teixitRepository.findByNom(nom);
    }

    /** Llista de teixits */
    @Override
    public List<Teixit> getTeixitsList() {
        return teixitRepository.findAll();
    }

    /** Llista de teixits d'un proveïdor */
    @Override
    public List<Teixit> getTeixitsListByProveidor(Long idProveidor) {
        return teixitRepository.findByProveidor(idProveidor);
    }

    /** Afegir o modificar teixit */
    @Override
    public Teixit save(Teixit teixit) {

        Teixit t = teixitRepository.save(teixit);

        if(t.getReferencia() == null && t.getId() > 0) {
            //referència autogenerada a partir de l'identificador
            String referencia = "T" + String.format("%05d", t.getId());
            t.setReferencia(referencia);
            return teixitRepository.save(t);
        }

        return t;
        
    }

    /** Eliminar teixit */
    @Override
    public void remove(Long id) {
        teixitRepository.deleteById(id);        
    }

    /** Booleà que indica si ja existeix el nom de teixit */
    @Override
    public boolean existsNom(String nom) {
        return teixitRepository.findByNom(nom)!=null;
    }

    /** Cerca a la taula */
    @Override
    public List<Teixit> findByKeyword(String keyword) {
        return teixitRepository.findByKeyword(keyword);
    }

    /** Paginació de la taula */
    @Override
    public Page<Teixit> findPaginated(int pageNo, int pageSize) {
        Pageable pageable = PageRequest.of(pageNo - 1, pageSize);
        return this.teixitRepository.findAll(pageable);
    }

    /** Afegir una composició al teixit */
    @Override
    public Teixit saveComposicio(Teixit teixit, Composicio composicio) {

        composicio.setTeixit(teixit);
        Composicio composicioTeixit = composicioRepository.save(composicio);

        List<Composicio> materialsComposicio = teixit.getMaterialsComposicio();
        materialsComposicio.add(composicioTeixit);
        teixit.setMaterialsComposicio(materialsComposicio);

        return teixitRepository.save(teixit);
    }

    /** Eliminar una composicio del teixit */
    @Override
    public void removeComposicio(Long id) {
        composicioRepository.deleteById(id);
        
    }

    /** Booleà que indica si el material a afegir a la composició del teixit 
     * ja existeix per aquell teixit */
    @Override
    public boolean hasMaterial(List<Composicio> materialsComposicio, Long id_material) {
        
        for (Composicio composicio : materialsComposicio) {
            if(composicio.getMaterial().getId() == id_material) {
                return true;
            }
        }
        return false;
    }    

    /** Booleà que indica si el teixit té escandalls associats */
    @Override
    public boolean hasEscandalls(Long id) {
        return !findById(id).getEscandalls().isEmpty();
    }
    
}
