package com.uoc.textiller.service;

import java.util.List;

import com.uoc.textiller.model.Usuari;
import com.uoc.textiller.repository.UsuariRepository;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Service;

/**
 * Implementació dels mètodes d'UsuariService
 * @author lrodriguezmos@uoc
 */
@Service
public class UsuariServiceImpl implements UsuariService {
    
    @Autowired
    UsuariRepository usuariRepository;

    @Autowired 
    BCryptPasswordEncoder encoder;

    public UsuariServiceImpl(UsuariRepository usuariRepository) {
        this.usuariRepository = usuariRepository;
    }

    /** Cerca per ID */
    @Override
    public Usuari findById(Long id) {
        return usuariRepository.findById(id).get();
    }

    /** Cerca per nom */
    @Override
    public Usuari findByNom(String nom) {
        return usuariRepository.findByNom(nom);
    }

    /** Llista d'usuaris */
    @Override
    public List<Usuari> getUsuarisList() {

        return usuariRepository.findAll();
    }

    /** Afegir o modificar usuari */
    @Override
    public Usuari save(Usuari usuari) {
        
        usuari.setPassword(encoder.encode(usuari.getPassword()));
        return usuariRepository.save(usuari);
    }

    /** Eliminar usuari */
    @Override
    public void remove(Long id) {
        usuariRepository.deleteById(id);        
    }

    /** Booleà que indica si ja existeix el nom d'usuari */
    @Override
    public boolean existsNom(String nom) {
        return usuariRepository.findByNom(nom)!=null;
    }

    
    /** Cerca a la taula */
    @Override
    public List<Usuari> findByKeyword(String keyword) {
        return usuariRepository.findByKeyword(keyword);
    }

    /** Paginació de la taula */
    @Override
    public Page<Usuari> findPaginated(int pageNo, int pageSize) {
        Pageable pageable = PageRequest.of(pageNo - 1, pageSize);
        return this.usuariRepository.findAll(pageable);
    }

    /** Booleà que indica si l'usuari té escandalls associats */
    @Override
    public boolean hasEscandalls(Long id) {
        return !findById(id).getEscandalls().isEmpty();
    }
}
