package com.uoc.textiller;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotEquals;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

import com.uoc.textiller.model.Proveidor;
import com.uoc.textiller.model.Teixit;
import com.uoc.textiller.model.Usuari;
import com.uoc.textiller.model.Escandall;
import com.uoc.textiller.model.EscandallFornitura;
import com.uoc.textiller.model.EscandallProces;
import com.uoc.textiller.model.EscandallTeixit;
import com.uoc.textiller.model.Fornitura;
import com.uoc.textiller.model.Proces;
import com.uoc.textiller.model.enums.EstatsUsuari;
import com.uoc.textiller.model.enums.LligatsTeixit;
import com.uoc.textiller.model.enums.RolsUsuari;
import com.uoc.textiller.repository.ProveidorRepository;
import com.uoc.textiller.repository.TeixitRepository;
import com.uoc.textiller.repository.UsuariRepository;
import com.uoc.textiller.repository.EscandallRepository;
import com.uoc.textiller.repository.FornituraRepository;
import com.uoc.textiller.repository.ProcesRepository;

import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.jdbc.AutoConfigureTestDatabase;
import org.springframework.boot.test.autoconfigure.jdbc.AutoConfigureTestDatabase.Replace;
import org.springframework.boot.test.autoconfigure.orm.jpa.DataJpaTest;

@DataJpaTest
@AutoConfigureTestDatabase(replace = Replace.NONE)
public class TestEscandallJPA {

    @Autowired
    private EscandallRepository repoEscandall;

    @Autowired
    private UsuariRepository repoUsuari;

    @Autowired
    private ProveidorRepository repoProveidor;

    @Autowired
    private TeixitRepository repoTeixit;

    @Autowired
    private FornituraRepository repoFornitura;

    @Autowired
    private ProcesRepository repoProces;

    public Usuari defaultUsuari(){
        //usuari per defecte pels escandalls
        Usuari usuari = new Usuari();
        usuari.setNom("usuari1");
        usuari.setPassword("password");
        usuari.setEstat(EstatsUsuari.ACTIU);
        usuari.setRol(RolsUsuari.ROLE_CONFECCIONISTA);
        return repoUsuari.save(usuari);
    }

    public Proveidor defaultProveidor(String nom){
        //proveidor per defecte pels teixits i fornitures dels escandalls
        Proveidor proveidor = new Proveidor();
        proveidor.setNom(nom);
        proveidor.setTelefon("123");
        proveidor.setEmail("email@email.com");
        return repoProveidor.save(proveidor);
    }

    public Teixit defaultTeixit(){
        Proveidor proveidor = defaultProveidor("prov1");
        //teixit per defecte pels escandalls (materials)
        Teixit teixit = new Teixit();
        teixit.setNom("Teixit1");
        teixit.setProveidor(proveidor);
        teixit.setCost(BigDecimal.valueOf(1));
        teixit.setEstoc(BigDecimal.valueOf(1));
        teixit.setAmple(BigDecimal.valueOf(1));
        teixit.setLligat(LligatsTeixit.PUNT);
        return repoTeixit.save(teixit);
    }

    public Fornitura defaultFornitura(){
        Proveidor proveidor = defaultProveidor("prov2");
        //fornitura per defecte pels escandalls (materials)
        Fornitura fornitura = new Fornitura();
        fornitura.setNom("Fornitura1");
        fornitura.setProveidor(proveidor);
        fornitura.setCost(BigDecimal.valueOf(1));
        fornitura.setEstoc(1);

        return repoFornitura.save(fornitura);
    }

    public Proces defaultProces(){
        //procés per defecte pels escandalls
        Proces proces = new Proces();
        proces.setNom("Proces1");
        proces.setCost(BigDecimal.valueOf(1));

        return repoProces.save(proces);
    }

    /**TEST: Afegir dos escandalls diferents a la base de dades 
     * 
     * AVÍS: és necessari tenir la base de dades buida per a la correcta execució
    */
    @Test
    public void testAfegirEscandalls() {

        Usuari usuari = defaultUsuari();

        Escandall escandall1 = new Escandall();
        escandall1.setNom("Escandall1");
        escandall1.setUsuari(usuari);

        Escandall escandall2 = new Escandall();
        escandall2.setNom("Escandall2");
        escandall2.setUsuari(usuari);

        repoEscandall.save(escandall1);
        repoEscandall.save(escandall2);

        repoEscandall.flush();

        //resultat esperat 1: la cerca per nom de "Escandall1" hauria de retornar un escandall vàlid
        assertNotEquals(null, repoEscandall.findByNom("Escandall1"));
        //resultat esperat 2: la cerca per nom de "Escandall2" hauria de retornar un escandall vàlid
        assertNotEquals(null, repoEscandall.findByNom("Escandall2"));
        //resultat esperat 3: el total de escandalls hauria de ser 2
        assertEquals(2, repoEscandall.findAll().size());

    }

    /**TEST: Editar un escandall existent a la base de dades */
    @Test
    public void testEditarEscandall() {

        Usuari usuari = defaultUsuari();

        Escandall escandall1 = new Escandall();
        escandall1.setNom("Escandall1");
        escandall1.setUsuari(usuari);

        Escandall escandall = repoEscandall.save(escandall1);
        Long idEscandall = escandall.getId();

        Optional<Escandall> escandallO = repoEscandall.findById(idEscandall);
        Escandall escandallEditar = escandallO.get();
        escandallEditar.setDescripcio("descripcio");
        Escandall escandallEditat = repoEscandall.save(escandallEditar);
               
        repoEscandall.flush();
        
        //resultat esperat: l'Escandall1 hauria de tenir descripció
        assertEquals("descripcio", escandallEditat.getDescripcio());

    }

    /**TEST: Eliminar un escandall existent de la base de dades */
    @Test
    public void testEliminarEscandall() {

        Usuari usuari = defaultUsuari();

        Escandall escandall1 = new Escandall();
        escandall1.setNom("Escandall1");
        escandall1.setUsuari(usuari);
        

        Escandall escandallEliminar = repoEscandall.save(escandall1);

        Long idEscandall = escandallEliminar.getId();

        repoEscandall.deleteById(idEscandall);
      
        repoEscandall.flush();

        //resultat esperat: cercar per nom l'escandall eliminat "Escandall1" hauria de retornar null
        assertEquals(null, repoEscandall.findByNom("Escandall1"));
    }

    

    /**TEST: Cercar escandalls a partir d'una paraula clau */
    @Test
    public void testCercarEscandalls() {

        Usuari usuari = defaultUsuari();

        Escandall escandall1 = new Escandall();
        escandall1.setNom("Escandall1");
        escandall1.setUsuari(usuari);
        escandall1.setDescripcio("descripcio");

        Escandall escandall2 = new Escandall();
        escandall2.setNom("Escandall2");
        escandall2.setUsuari(usuari);
        escandall2.setDescripcio("descripcio");

        Escandall escandall3 = new Escandall();
        escandall3.setUsuari(usuari);
        escandall3.setNom("Escandall3");

        repoEscandall.save(escandall1);
        repoEscandall.save(escandall2);
        repoEscandall.save(escandall3);
      
        repoEscandall.flush();


        List<Escandall> escandallList = repoEscandall.findByKeyword("Escandall");
        List<Escandall> descripcioList = repoEscandall.findByKeyword("descripcio");

        //resultat esperat 1: cercar per "Escandall" hauria de retornar una llista de 3 escandalls
        assertEquals(3, escandallList.size());
        //resultat esperat 2: cercar per "descripcio" hauria de retornar una llista d'1 escandall
        assertEquals(2, descripcioList.size());
    }
    
    /**TEST: Afegir teixits, fornitures i processos a l'escandall */
    @Test
    public void testAfegirPartsEscandall() {

        Usuari usuari = defaultUsuari();
        Teixit teixit = defaultTeixit();
        Fornitura fornitura = defaultFornitura();
        Proces proces = defaultProces();

        Escandall escandall = new Escandall();
        escandall.setNom("Escandall");
        escandall.setUsuari(usuari);

        repoEscandall.save(escandall);

        repoEscandall.flush();

        EscandallTeixit teixitEscandall = new EscandallTeixit();
        teixitEscandall.setEscandall(escandall);
        teixitEscandall.setTeixit(teixit);
        teixitEscandall.setTotal(BigDecimal.valueOf(1));

        EscandallFornitura fornituraEscandall = new EscandallFornitura();
        fornituraEscandall.setEscandall(escandall);
        fornituraEscandall.setFornitura(fornitura);
        fornituraEscandall.setTotal(1);

        EscandallProces procesEscandall = new EscandallProces();
        procesEscandall.setEscandall(escandall);
        procesEscandall.setProces(proces);
        procesEscandall.setTotal(1);

        List<EscandallTeixit> teixitsEscandall = new ArrayList<>();
        teixitsEscandall.add(teixitEscandall);
        escandall.setTeixits(teixitsEscandall);

        List<EscandallFornitura> fornituresEscandall = new ArrayList<>();
        fornituresEscandall.add(fornituraEscandall);
        escandall.setFornitures(fornituresEscandall);

        List<EscandallProces> processosEscandall = new ArrayList<>();
        processosEscandall.add(procesEscandall);
        escandall.setProcessos(processosEscandall);

        repoEscandall.save(escandall);
        repoEscandall.flush();

        Escandall escandallAssociat = repoEscandall.findByNom("Escandall");
        

        //resultat esperat 1: l'escandall ha tenir un teixit associat
        assertEquals(1, escandallAssociat.getTeixits().size());

        //resultat esperat 2: l'escandall ha tenir una fornitura associada
        assertEquals(1, escandallAssociat.getFornitures().size());

        //resultat esperat 3: l'escandall ha tenir un procés associat
        assertEquals(1, escandallAssociat.getProcessos().size());

    }
}
