package com.uoc.textiller;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotEquals;

import java.util.List;
import java.util.Optional;

import com.uoc.textiller.model.Material;
import com.uoc.textiller.repository.MaterialRepository;

import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.jdbc.AutoConfigureTestDatabase;
import org.springframework.boot.test.autoconfigure.jdbc.AutoConfigureTestDatabase.Replace;
import org.springframework.boot.test.autoconfigure.orm.jpa.DataJpaTest;

@DataJpaTest
@AutoConfigureTestDatabase(replace = Replace.NONE)
public class TestMaterialJPA {

    @Autowired
    private MaterialRepository repoMaterial;

    /**TEST: Afegir dos materials diferents a la base de dades 
     * 
     * AVÍS: és necessari tenir la base de dades buida per a la correcta execució
    */
    @Test
    public void testAfegirMaterials() {

        Material material1 = new Material();
        material1.setNom("Llana");
        material1.setCodi("WV");

        Material material2 = new Material();
        material2.setNom("Cotó");
        material2.setCodi("CO");

        repoMaterial.save(material1);
        repoMaterial.save(material2);

        repoMaterial.flush();

        //resultat esperat 1: la cerca per nom de "Llana" hauria de retornar un material vàlid
        assertNotEquals(null, repoMaterial.findByNom("Llana"));
        //resultat esperat 2: la cerca per nom de "Cotó" hauria de retornar un material vàlid
        assertNotEquals(null, repoMaterial.findByNom("Cotó"));
        //resultat esperat 3: el total de materials hauria de ser 2
        assertEquals(2, repoMaterial.findAll().size());

    }

    /**TEST: Editar un material existent a la base de dades */
    @Test
    public void testEditarMaterial() {

        Material material1 = new Material();
        material1.setNom("Llana");
        material1.setCodi("WO");

        Material material = repoMaterial.save(material1);
        Long idMaterial = material.getId();

        Optional<Material> materialO = repoMaterial.findById(idMaterial);
        Material materialEditar = materialO.get();
        materialEditar.setCodi("WV");
        Material materialEditat = repoMaterial.save(materialEditar);
               
        repoMaterial.flush();
        
        //resultat esperat: el material Llana hauria de tenir el codi WV en comptes de WO
        assertEquals("WV", materialEditat.getCodi());

    }

    /**TEST: Eliminar un material existent de la base de dades */
    @Test
    public void testEliminarMaterial() {

        Material material1 = new Material();
        material1.setNom("Llana");
        material1.setCodi("WO");

        Material materialEliminar = repoMaterial.save(material1);

        Long idMaterial = materialEliminar.getId();

        repoMaterial.deleteById(idMaterial);
      
        repoMaterial.flush();

        //resultat esperat: cercar per nom el material eliminat "Llana" hauria de retornar null
        assertEquals(null, repoMaterial.findByNom("Llana"));
    }

    

    /**TEST: Cercar materials a partir d'una paraula clau */
    @Test
    public void testCercarMaterials() {

        Material material1 = new Material();
        material1.setNom("Llana verge");
        material1.setCodi("WO");

        Material material2 = new Material();
        material2.setNom("Cotó");
        material2.setCodi("CO");

        Material material3 = new Material();
        material3.setNom("Llana regenerada");
        material3.setCodi("WV");

        repoMaterial.save(material1);
        repoMaterial.save(material2);
        repoMaterial.save(material3);
      
        repoMaterial.flush();


        List<Material> llanaList = repoMaterial.findByKeyword("Lana");
        List<Material> cotoList = repoMaterial.findByKeyword("Cotó");

        //resultat esperat 1: cercar per "Llana" hauria de retornar una llista de 2 materials
        assertEquals(2, llanaList.size());
        //resultat esperat 2: cercar per "Cotó" hauria de retornar una llista d'1 material
        assertEquals(1, cotoList.size());
    }
    
}
