package com.uoc.textiller;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotEquals;

import java.math.BigDecimal;
import java.util.List;
import java.util.Optional;

import com.uoc.textiller.model.Proces;
import com.uoc.textiller.repository.ProcesRepository;

import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.jdbc.AutoConfigureTestDatabase;
import org.springframework.boot.test.autoconfigure.jdbc.AutoConfigureTestDatabase.Replace;
import org.springframework.boot.test.autoconfigure.orm.jpa.DataJpaTest;

@DataJpaTest
@AutoConfigureTestDatabase(replace = Replace.NONE)
public class TestProcesJPA {

    @Autowired
    private ProcesRepository repoProces;

    /**TEST: Afegir dos processos diferents a la base de dades 
     * 
     * AVÍS: és necessari tenir la base de dades buida per a la correcta execució
    */
    @Test
    public void testAfegirProcessos() {

        Proces proces1 = new Proces();
        proces1.setNom("Confecció");
        proces1.setCost(BigDecimal.valueOf(1));

        Proces proces2 = new Proces();
        proces2.setNom("Tall");
        proces2.setCost(BigDecimal.valueOf(1));

        repoProces.save(proces1);
        repoProces.save(proces2);

        repoProces.flush();

        //resultat esperat 1: la cerca per nom de "Confecció" hauria de retornar un procés vàlid
        assertNotEquals(null, repoProces.findByNom("Confecció"));
        //resultat esperat 2: la cerca per nom de "Tall" hauria de retornar un procés vàlid
        assertNotEquals(null, repoProces.findByNom("Tall"));
        //resultat esperat 3: el total de processos hauria de ser 2
        assertEquals(2, repoProces.findAll().size());

    }

    /**TEST: Editar un procés existent a la base de dades */
    @Test
    public void testEditarProces() {

        Proces proces1 = new Proces();
        proces1.setNom("Confecció");
        proces1.setCost(BigDecimal.valueOf(1));

        Proces proces = repoProces.save(proces1);
        Long idProces = proces.getId();

        Optional<Proces> procesO = repoProces.findById(idProces);
        Proces procesEditar = procesO.get();
        procesEditar.setCost(BigDecimal.valueOf(5));
        Proces procesEditat = repoProces.save(procesEditar);
               
        repoProces.flush();
        
        //resultat esperat: el procés Confecció hauria de tenir un cost de 5 en comptes de 1
        assertEquals(BigDecimal.valueOf(5), procesEditat.getCost());

    }

    /**TEST: Eliminar un procés existent de la base de dades */
    @Test
    public void testEliminarProces() {

        Proces proces1 = new Proces();
        proces1.setNom("Confecció");
        proces1.setCost(BigDecimal.valueOf(1));

        Proces procesEliminar = repoProces.save(proces1);

        Long idProces = procesEliminar.getId();

        repoProces.deleteById(idProces);
      
        repoProces.flush();

        //resultat esperat: cercar per nom el proces eliminat "Confecció" hauria de retornar null
        assertEquals(null, repoProces.findByNom("Confecció"));
    }

    

    /**TEST: Cercar processos a partir d'una paraula clau */
    @Test
    public void testCercarProcessos() {

        Proces proces1 = new Proces();
        proces1.setNom("Confecció a mà");
        proces1.setCost(BigDecimal.valueOf(1));

        Proces proces2 = new Proces();
        proces2.setNom("Tall");
        proces2.setCost(BigDecimal.valueOf(1));

        Proces proces3 = new Proces();
        proces3.setNom("Confecció a màquina");
        proces3.setCost(BigDecimal.valueOf(1));

        repoProces.save(proces1);
        repoProces.save(proces2);
        repoProces.save(proces3);
      
        repoProces.flush();


        List<Proces> confeccioList = repoProces.findByKeyword("Confecció");
        List<Proces> tallList = repoProces.findByKeyword("Tall");

        //resultat esperat 1: cercar per "Confecció" hauria de retornar una llista de 2 processos
        assertEquals(2, confeccioList.size());
        //resultat esperat 2: cercar per "Tall" hauria de retornar una llista d'1 procés
        assertEquals(1, tallList.size());
    }
    
}
