package com.uoc.textiller;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotEquals;

import java.math.BigDecimal;
import java.util.List;
import java.util.Optional;

import com.uoc.textiller.model.Proveidor;
import com.uoc.textiller.model.Teixit;
import com.uoc.textiller.model.enums.LligatsTeixit;
import com.uoc.textiller.repository.ProveidorRepository;
import com.uoc.textiller.repository.TeixitRepository;

import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.jdbc.AutoConfigureTestDatabase;
import org.springframework.boot.test.autoconfigure.jdbc.AutoConfigureTestDatabase.Replace;
import org.springframework.boot.test.autoconfigure.orm.jpa.DataJpaTest;

@DataJpaTest
@AutoConfigureTestDatabase(replace = Replace.NONE)
public class TestTeixitJPA {

    @Autowired
    private TeixitRepository repoTeixit;

    @Autowired
    private ProveidorRepository repoProveidor;

    public Proveidor defaultProveidor(){
        //necessitem un proveïdor per afegir teixits
        Proveidor proveidor = new Proveidor();
        proveidor.setNom("Prov1");
        proveidor.setTelefon("123");
        proveidor.setEmail("email@email.com");
        return repoProveidor.save(proveidor);
    }

    /**TEST: Afegir dos teixits diferents a la base de dades 
     * 
     * AVÍS: és necessari tenir la base de dades buida per a la correcta execució
    */
    @Test
    public void testAfegirTeixits() {

        Proveidor proveidor = defaultProveidor();

        Teixit teixit1 = new Teixit();
        teixit1.setNom("Teixit1");
        teixit1.setProveidor(proveidor);
        teixit1.setCost(BigDecimal.valueOf(1));
        teixit1.setEstoc(BigDecimal.valueOf(1));
        teixit1.setAmple(BigDecimal.valueOf(1));
        teixit1.setLligat(LligatsTeixit.PUNT);

        Teixit teixit2 = new Teixit();
        teixit2.setNom("Teixit2");
        teixit2.setProveidor(proveidor);
        teixit2.setCost(BigDecimal.valueOf(2));
        teixit2.setEstoc(BigDecimal.valueOf(2));
        teixit2.setAmple(BigDecimal.valueOf(2));
        teixit2.setLligat(LligatsTeixit.SARJA);

        repoTeixit.save(teixit1);
        repoTeixit.save(teixit2);

        repoTeixit.flush();

        //resultat esperat 1: la cerca per nom de "Teixit1" hauria de retornar un teixit vàlid
        assertNotEquals(null, repoTeixit.findByNom("Teixit1"));
        //resultat esperat 2: la cerca per nom de "Teixit2" hauria de retornar un teixit vàlid
        assertNotEquals(null, repoTeixit.findByNom("Teixit2"));
        //resultat esperat 3: el total de teixits hauria de ser 2
        assertEquals(2, repoTeixit.findAll().size());

    }

    /**TEST: Editar un teixit existent a la base de dades */
    @Test
    public void testEditarTeixit() {

        Proveidor proveidor = defaultProveidor();

        Teixit teixit1 = new Teixit();
        teixit1.setNom("Teixit1");
        teixit1.setProveidor(proveidor);
        teixit1.setCost(BigDecimal.valueOf(1));
        teixit1.setEstoc(BigDecimal.valueOf(1));
        teixit1.setAmple(BigDecimal.valueOf(1));
        teixit1.setLligat(LligatsTeixit.PUNT);

        Teixit teixit = repoTeixit.save(teixit1);
        Long idTeixit = teixit.getId();

        Optional<Teixit> teixitO = repoTeixit.findById(idTeixit);
        Teixit teixitEditar = teixitO.get();
        teixitEditar.setLligat(LligatsTeixit.TAFETA);
        Teixit teixitEditat = repoTeixit.save(teixitEditar);
               
        repoTeixit.flush();
        
        //resultat esperat: el Teixit1 hauria de tenir lligat de tipus tafetà en comptes de punt
        assertEquals(LligatsTeixit.TAFETA, teixitEditat.getLligat());

    }

    /**TEST: Eliminar un teixit existent de la base de dades */
    @Test
    public void testEliminarTeixit() {

        Proveidor proveidor = defaultProveidor();

        Teixit teixit1 = new Teixit();
        teixit1.setNom("Teixit1");
        teixit1.setProveidor(proveidor);
        teixit1.setCost(BigDecimal.valueOf(1));
        teixit1.setEstoc(BigDecimal.valueOf(1));
        teixit1.setAmple(BigDecimal.valueOf(1));
        teixit1.setLligat(LligatsTeixit.PUNT);

        Teixit teixitEliminar = repoTeixit.save(teixit1);

        Long idTeixit = teixitEliminar.getId();

        repoTeixit.deleteById(idTeixit);
      
        repoTeixit.flush();

        //resultat esperat: cercar per nom el teixit eliminat "Teixit1" hauria de retornar null
        assertEquals(null, repoTeixit.findByNom("Teixit1"));
    }

    

    /**TEST: Cercar teixits a partir d'una paraula clau */
    @Test
    public void testCercarTeixits() {

        Proveidor proveidor = defaultProveidor();

        Teixit teixit1 = new Teixit();
        teixit1.setNom("Teixit1");
        teixit1.setProveidor(proveidor);
        teixit1.setCost(BigDecimal.valueOf(1));
        teixit1.setEstoc(BigDecimal.valueOf(1));
        teixit1.setAmple(BigDecimal.valueOf(1));
        teixit1.setLligat(LligatsTeixit.PUNT);

        Teixit teixit2 = new Teixit();
        teixit2.setNom("Teixit2");
        teixit2.setProveidor(proveidor);
        teixit2.setCost(BigDecimal.valueOf(2));
        teixit2.setEstoc(BigDecimal.valueOf(2));
        teixit2.setAmple(BigDecimal.valueOf(2));
        teixit2.setLligat(LligatsTeixit.SARJA);

        Teixit teixit3 = new Teixit();
        teixit3.setNom("Teixit3");
        teixit3.setProveidor(proveidor);
        teixit3.setCost(BigDecimal.valueOf(2));
        teixit3.setEstoc(BigDecimal.valueOf(2));
        teixit3.setAmple(BigDecimal.valueOf(2));
        teixit3.setLligat(LligatsTeixit.SARJA);

        repoTeixit.save(teixit1);
        repoTeixit.save(teixit2);
        repoTeixit.save(teixit3);
      
        repoTeixit.flush();


        List<Teixit> teixitList = repoTeixit.findByKeyword("Teixit");
        List<Teixit> sarjaList = repoTeixit.findByKeyword("Sarja");

        //resultat esperat 1: cercar per "Teixit" hauria de retornar una llista de 3 teixits
        assertEquals(3, teixitList.size());
        //resultat esperat 2: cercar per "Sarja" hauria de retornar una llista d'1 teixit
        assertEquals(2, sarjaList.size());
    }
    
}
